#include <pvm3.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <defs.h>
#include "pvmfarm.h"

int stid = 0;

int SearchForServer(char *name)
{
	int ret;
	int	ntasks,i;
	struct taskinfo *taskp;
	
	ret = pvm_tasks(0,&ntasks,&taskp);
	if (ret != 0)
	{
		printf("Error in SearchForServer(): pvm_tasks returns %d\n",ret);	
		pvm_exit();
		exit(1);
	}
	// So now the taskp contains the information
	for (i=0; i< ntasks;i++)
	{
		if (!strcmp(name,taskp[i].ti_a_out))
		{ //matches perfectly so we have the name
			return (taskp[i].ti_tid);
		}
	}	
	return (-1);
}

int GetTaskName(int tid,char *name)
{
	// this routine can be used to get a.out name of a task specified 
	// by a tid.

	int ret;
	int	ntasks,i;
	struct taskinfo *taskp;
	
	ret = pvm_tasks(0,&ntasks,&taskp);
	if (ret != 0)
	{
		printf("Error in SearchForServer(): pvm_tasks returns %d\n",ret);	
		pvm_exit();
		exit(1);
	}
	// So now the taskp contains the information
	for (i=0; i< ntasks;i++)
	{
		if (tid == taskp[i].ti_tid)
		{
			// found the task tid 
			strcpy(name,taskp[i].ti_a_out);
			return (1);
		}
	}
	return (-1); // not found 
}

int pvm_start_farmd(void)
{
int ret;

 	ret = pvm_spawn(SERVER_NAME,(char**)0, 0, "", 1, &stid);
	if (ret < 0)
		return (ERROR);
	else 
		return (OK);
}
	
int pvm_farm_init(char *farm_name)
{
	int  tid,ret,bid,opcode;

	//cc = pvm_spawn(SERVER_NAME, (char**)0, 0, "", 1, &tid);
	tid = pvm_mytid();

	if(stid == 0) // server has been found or not
	{
		if((stid = SearchForServer(SERVER_NAME)) > 0)
		{
			//printf("pvm_farm_init(\"%s\"):Found Server : stid = %x, tid = %x\n",farm_name,stid,tid);
		}
		else
		{
			printf("pvm_farm_init(\"%s\"):Couldn't locate the server task\n",farm_name);
			pvm_exit();
			exit(1);
		}
	}

	bid = pvm_initsend(PvmDataDefault); // create a buffer
	pvm_setsbuf(bid); // set this as active send buffer
	opcode = INIT_FARM; // opcode for server
	pvm_pkint(&opcode, 1, 1); // pack the data
	pvm_pkstr(farm_name); // pack the farm name in the buffer
	pvm_send(stid, DEF_MSGTAG); // message of type 1.

	pvm_recv(stid,DEF_MSGTAG);
	pvm_upkint(&ret,1,1);

	printf("pvm_init_farm(\"%s\") ; returns %d\n",farm_name, ret);
	if(ret != ERROR)
		return (OK);
	else
		return (ERROR);
	
}

int pvm_get_worker_class_id(char *farm_name, char *wcname)
{
	int bid, opcode, ret, tid;
	
	tid = pvm_mytid(); // locate my own tid

	if(stid == 0) // server has been found or not
	{
		if((stid = SearchForServer(SERVER_NAME)) > 0)	
		{
			printf("Found Server : stid = %x, tid = %x\n",stid,tid);
		}
		else
		{
			printf("Couldn't locate the server task\n");
			pvm_exit();
			exit(1);
		}
	}

	bid = pvm_initsend(PvmDataDefault); // make buffer to send data
	pvm_setsbuf(bid); // explictly declare this buffer id to be used for
			// sending. Make this active send buffer.
	
	opcode = GET_WORKER_CLASS_ID; // operation that the server should perform
	pvm_pkint(&opcode, 1, 1); // pack this item 

	pvm_pkstr(farm_name); // pack the farm name in the buffer
	pvm_pkstr(wcname); // pack the worker class name
	pvm_send(stid, DEF_MSGTAG); // message of type 1.
	// Now wait for a message from the Server. The message contains wcid.

	pvm_recv(stid,DEF_MSGTAG);
	pvm_upkint(&ret,1,1);

	printf("pvm_get_worker_class_id(\"%s\",\"%s\" );  returns %d\n",farm_name, wcname, ret);
	return (ret); // return whatever is recd. wcid or ERROR
}

int pvm_send_work_packet(int wcid)
{
int bufid,msgbufid;
int opcode;
	

	bufid = pvm_mkbuf(PvmDataDefault);	

	msgbufid = pvm_setsbuf(bufid); // gives the old bufid which has the 
			// message (made by the user)
	if(stid == 0)
		return (ERROR); // the server has not been located yet??

	opcode = SEND_WORK_PACKET; 
	pvm_pkint(&opcode, 1, 1); // pack the request

	pvm_pkint(&wcid, 1, 1); // pack the worker class id

	pvm_send(stid, DEF_MSGTAG); // send the request to the server

	pvm_setsbuf(msgbufid); // users work packet is next to go
	
	pvm_send(stid, DEF_MSGTAG); // send the work packet to the server
	//printf("pvm_send_work_packet(): Sent buffer %d\n", msgbufid);

	return (OK); // fine everything went well

}

int pvm_recv_reply_packet(int wcid) 
{
int bufid;
	bufid = pvm_recv(-1, wcid); // receive a message from any worker
		// task that has msgtag = worker class id 
	if (bufid < 0) 
		return ERROR;
	else
		return bufid; // bufid or the Error.

}

int pvm_farm_terminate(char *farm_name) 
{
int	bufid, ret, opcode;

	bufid = pvm_mkbuf(PvmDataDefault);	

	pvm_setsbuf(bufid); // sets this as the active send buffer
	if(stid == 0)
		return (ERROR); // the server has not been located yet??

	opcode = FARM_TERMINATE;  // destroy everything associated with 
		// this farm. Its use is over!
	pvm_pkint(&opcode, 1, 1); // pack the request

	pvm_pkstr(farm_name); // pack  the farm name

	pvm_send(stid, DEF_MSGTAG); // send the request to the server

	pvm_freebuf(bufid);

	pvm_recv(stid,DEF_MSGTAG);
	pvm_upkint(&ret,1,1);

	printf("pvm_farm_terminate (\"%s\");  returns %d\n",farm_name, ret);
	return (ret); // return whatever is recd. NO_SUCH_FARM or NOT_FARM_OWNER
			// or OK
}

int pvm_stop_farmd(void) 
{
int	bufid, ret, opcode;

	bufid = pvm_mkbuf(PvmDataDefault);	

	pvm_setsbuf(bufid); // sets this as the active send buffer
	if(stid == 0)
		return (ERROR); // the server has not been located yet??

	opcode = STOP_FARMD;  // stop the farmer daemon
		// Its use is over!
	pvm_pkint(&opcode, 1, 1); // pack the request


	pvm_send(stid, DEF_MSGTAG); // send the request to the server

	pvm_freebuf(bufid);

	return (OK); 
}
	
	
