#include <pvm3.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "defs.h"
#include "pvmfarm.h"

extern int stid;
extern int SearchForServer(char *name);

int farmer_tid; // tid of the farmer task
char farmer_name[NAMELEN];
char worker_class_name[NAMELEN];
int worker_class_id; // wc id 

int pvm_init_worker_class(char *farm_name, char *wcname)
{
	int bid, opcode, ret, tid;
	
	tid = pvm_mytid(); // locate my own tid
	strcpy(farmer_name, farm_name); // make a local copy
	strcpy(worker_class_name, wcname); // make a local copy 

	if(stid == 0) // server has been found or not
	{
		if((stid = SearchForServer(SERVER_NAME)) > 0)	
		{
			//printf("Found Server : stid = %x, tid = %x\n",stid,tid);
		}
		else
		{
			printf("Couldn't locate the server task\n");
			pvm_exit();
			exit(1);
		}
	}


	bid = pvm_initsend(PvmDataDefault); // make buffer to send data
	pvm_setsbuf(bid); // explictly declare this buffer id to be used for
			// sending. Make this active send buffer.
	
	opcode = INIT_WORKER_CLASS; // operation that the server should perform
	pvm_pkint(&opcode, 1, 1); // pack this item 

	pvm_pkstr(farm_name); // pack the farm name in the buffer
	pvm_pkstr(wcname); // pack the worker class name
	pvm_send(stid, DEF_MSGTAG); // message of type 1.
	// Now wait for a message from the Server. The message contains wcid.

	pvm_recv(stid,DEF_MSGTAG);
	pvm_upkint(&ret,1,1);
	pvm_upkint(&farmer_tid, 1, 1); // get farmer's tid

	worker_class_id = ret; 
	printf("pvm_init_worker_class(\"%s\",\"%s\" );  returns %d and ftid = %x\n",farm_name, wcname, ret,farmer_tid);
	return (ret); // return whatever is recd. wcid or ERROR
}

int pvm_send_reply_packet(void)
{
	// we know the farmer tid, and the worker class id

	if(worker_class_id < 0)
	{
		return (ERROR); // cannot send wcid is bad
	}
	
	pvm_send(farmer_tid, worker_class_id); // send the message of
// type worker class id
	return (OK);
}

int pvm_recv_work_packet(void) 
{
int bufid,rbufid,opcode,x;
int info, size, msgtag, ctid;

	bufid = pvm_mkbuf(PvmDataDefault); // make buffer to send data
	pvm_setsbuf(bufid); // explictly declare this buffer id to be used for
			// sending. Make this active send buffer.
	
	opcode = RECV_WORK_PACKET; // operation that the server should perform
	pvm_pkint(&opcode, 1, 1); // pack this item 
	pvm_pkint(&worker_class_id, 1, 1); // pack the worker class id
	pvm_pkint(&farmer_tid, 1, 1); // pack the farmer tid

	pvm_send(stid,DEF_MSGTAG); // send the request to the server
	pvm_freebuf(bufid); // free the buffer
	
	rbufid = pvm_recv(stid, -1); // receive data from server

	if (rbufid < 0)
		return ERROR; // ERROR in receiving (pvm error)
	// Now check the message tag of the message received 
	info = pvm_bufinfo(rbufid, &size, &msgtag, &ctid); // size of message,
				// message tag, client tid (sender tasks tid)
	//printf("pvm_recv_work_packet(): received rbufid = %d, size = %d, msgtag = %d, ctid = %x\n", rbufid, size, msgtag, ctid);
	if (msgtag == FARM_HAS_TERMINATED)
		return PvmFarmHasTerminated; 
	else 
		return rbufid; // recv bufid 
	// This could be FARM_HAS_TERMINATED or bufid
}

