#include <pvm3.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "defs.h"
#include "pvmfarm.h"

extern int stid;
extern int SearchForServer(char *name);

int farmer_tid; // tid of the farmer task
char farmer_name[NAMELEN];
char worker_class_name[NAMELEN];
int worker_class_id; // wc id 

int pvm_init_worker_class(char *farm_name, char *wcname)
{
	int bid, opcode, ret, tid, cc;
	
	tid = pvm_mytid(); // locate my own tid
	ASSERT(tid);

	strcpy(farmer_name, farm_name); // make a local copy
	strcpy(worker_class_name, wcname); // make a local copy 

	if(stid == 0) // server has been found or not
	{
		if((stid = SearchForServer(SERVER_NAME)) > 0)	
		{
			//printf("Found Server : stid = %x, tid = %x\n",stid,tid);
		}
		else
		{
			printf("Couldn't locate the server task\n");
			pvm_exit();
			exit(1);
		}
	}


	bid = pvm_initsend(PvmDataDefault); // make buffer to send data
	ASSERT(bid);

	cc = pvm_setsbuf(bid); //explictly declare this buffer id to be used for
	ASSERT(cc);		
	
	opcode = INIT_WORKER_CLASS; // operation that the server should perform
	cc = pvm_pkint(&opcode, 1, 1); // pack this item 
	ASSERT(cc);	

	cc = pvm_pkstr(farm_name); // pack the farm name in the buffer
	ASSERT(cc);

	cc = pvm_pkstr(wcname); // pack the worker class name
	ASSERT(cc);

	cc = pvm_send(stid, DEF_MSGTAG); // message of type 1.
	ASSERT(cc);
	// Now wait for a message from the Server. The message contains wcid.

	cc = pvm_recv(stid,DEF_MSGTAG);
	ASSERT(cc);

	cc = pvm_upkint(&ret,1,1);
	ASSERT(cc);

	cc = pvm_upkint(&farmer_tid, 1, 1); // get farmer's tid
	ASSERT(cc);

	worker_class_id = ret; 
	//printf("pvm_init_worker_class(\"%s\",\"%s\" );  returns %d and ftid = %x\n",farm_name, wcname, ret,farmer_tid);
	return (ret); // return whatever is recd. wcid or ERROR
}

int pvm_send_reply_packet(void)
{ 
	int cc;
	// we know the farmer tid, and the worker class id

	if(worker_class_id < 0)
	{
		return (ERROR); // cannot send wcid is bad
	}
	
	cc = pvm_send(farmer_tid, worker_class_id); // send the message of
					// type worker class id
	ASSERT(cc);		 
	return (OK);
}

int pvm_recv_work_packet(void) 
{
int bufid,rbufid,opcode,x,cc;
int info, size, msgtag, ctid;

	bufid = pvm_mkbuf(PvmDataDefault); // make buffer to send data
	ASSERT(bufid);

	cc = pvm_setsbuf(bufid); //explictly declare this buffer id to be used 
			// for sending. Make this active send buffer.
	ASSERT(cc); 	// Check if < 0 

	opcode = RECV_WORK_PACKET; // operation that the server should perform
	cc = pvm_pkint(&opcode, 1, 1); // pack this item 
	ASSERT(cc); 	// Check if < 0 

	cc = pvm_pkint(&worker_class_id, 1, 1); // pack the worker class id
	ASSERT(cc); 	// Check if < 0 

	cc = pvm_send(stid,DEF_MSGTAG); // send the request to the server
	ASSERT(cc); 	// Check if < 0 

	cc = pvm_freebuf(bufid); // free the buffer
	ASSERT(cc); 	// Check if < 0 
	
	rbufid = pvm_recv(stid, -1); // receive data from server

	if (rbufid < 0)
		return ERROR; // ERROR in receiving (pvm error)
	// Now check the message tag of the message received 
	info = pvm_bufinfo(rbufid, &size, &msgtag, &ctid); // size of message,
				// message tag, client tid (sender tasks tid)
	//printf("pvm_recv_work_packet(): received rbufid = %d, size = %d, msgtag = %d, ctid = %x\n", rbufid, size, msgtag, ctid);
	ASSERT(info); 

	if (msgtag == FARM_HAS_TERMINATED)
		return PvmFarmHasTerminated; 
	else 
		return rbufid; // recv bufid 
	// This could be FARM_HAS_TERMINATED or bufid
}

// pvm_leave_farm is called when the worker wants disassociate itself from the
// farm.
int	pvm_leave_farm(void)
{
int bufid,rbufid,opcode,ret,cc;

	bufid = pvm_mkbuf(PvmDataDefault); // make buffer to send data
	cc = pvm_setsbuf(bufid); // explictly declare this buffer id to be used 
			// for sending. Make this active send buffer.
	ASSERT(cc); 	// Check if < 0 
	
	
	opcode = LEAVE_FARM; // operation that the server should perform
	cc = pvm_pkint(&opcode, 1, 1); // pack this item 
	ASSERT(cc); 	// Check if < 0 

	cc = pvm_pkint(&worker_class_id, 1, 1); // pack the worker class id
	ASSERT(cc); 	// Check if < 0 
	cc = pvm_pkint(&farmer_tid, 1, 1); // pack the farmer tid
	ASSERT(cc); 	// Check if < 0 

	cc = pvm_send(stid,DEF_MSGTAG); // send the request to the server
	ASSERT(cc); 	// Check if < 0 

	cc = pvm_freebuf(bufid); // free the buffer
	ASSERT(cc); 	// Check if < 0 
	
	rbufid = pvm_recv(stid, -1); // receive data from server

	if (rbufid < 0)
		return ERROR; // error in receiving the message

	cc = pvm_upkint(&ret, 1, 1); // get the return value from the server
	ASSERT(cc); 	// Check if < 0 

	if (ret == OK)
		return (OK);
	else
		return (ERROR); 
}
