#include "list.h"

Item::Item()
{
	next = (Item *) NULL;
}
Item::~Item()
{
// Nothing. Item is like a struct here	
}

List::List()
{ //Constructor. Initialise.
	first_rec = (Item *) NULL;
	last_rec = (Item *) NULL;
	no_of_items = 0;

}
List::~List()
{
//Destructor. Delete each item individually
Item *ptr,*next;
	ptr = first_rec;
	while (ptr != (Item *)NULL)
	{
		next = ptr->next; // it points to the next item
		delete ptr; // free the space
		ptr = next; // go on to the next, till the end when ptr= NULL
	}
}

List::Append(int val)
{
	Item *ptr;

	ptr = new Item(); // creates the new item
	
	if (ptr == (Item *) NULL)
	{
		printf("MEM ALLOCATION ERROR in APPEND. new Item() returns NULL\n");
		return (ERROR);
	}
	
	if (no_of_items == 0)
	{
		first_rec = ptr; // first rec points to the new item
		last_rec = first_rec; // there's only one item
	}
	else 
	{ // there are some items on the list
		last_rec->next = ptr; // Append to the end of the list
		last_rec = last_rec->next; //update the last_rec
	} //note last_rec->next = (Item *) NULL; is done in the Item::Item()
		
	last_rec->value = val; // put the value in the last item
	no_of_items ++; //in any case increment the no. of items.
	return (OK);
	
}

Item * List::Search(int val) 
{
	Item *ptr;
	
	ptr = first_rec; // start from the beginning of the list
	while (ptr != (Item *) NULL)
	{ // ptr points to something valid	
		if (ptr->value == val) // compare and check
		{
			return ptr; // return the item address - whole record
		}	
		else 
		{ // match not found. go on to the next item and check
			ptr = ptr -> next;
		}
	}
	// traversed the whole List and found no item which matches the value
	return (Item *) NULL; // Not found
}

List::Delete(int val) 
{
	Item *prev, *ptr;

	prev = (Item *) NULL; //points to the last item checked. NULL to start
	ptr = first_rec; // points to beginning of the List
	if (no_of_items == 0) // there's nothing to delete
		return OK; // fine I've done my job.

	while (ptr != (Item *) NULL) // check till there's no more to check
	{
		if(ptr -> value == val)
		{
		//matched. Delete this item now	
			if (prev != (Item *)NULL) // not the first item
			{
				prev -> next = ptr -> next; //skip this item
				if(prev -> next == (Item *) NULL) //last item
				{
					last_rec = prev; //update last rec	
				}
			} 
			else
			{ // this is the first item to be deleted
				first_rec = ptr -> next;
				if (first_rec == (Item *) NULL)
				{ // no items on the list
					last_rec = (Item *) NULL; //update
				}
			}
			
			// in either case, 
			delete (ptr); // delete the matched item
			no_of_items --; // and decrement the length of the List
			return (OK); // fine I've deleted the item
		} 
		// item does not match
		prev = ptr; // now I save the current item in prev and 
		ptr = ptr -> next; //go to the next item in the list
	}
	// No items found to delete
	return (FAILURE);
}


Item * List::Pop(void)
{ // Just remove the first item from the list
	Item *ptr;

	if (no_of_items == 0)
	{
		return (Item *) NULL; // There's no item on the list
	}
	
	ptr = first_rec; // take the first item
	first_rec = first_rec -> next; // it now points to the next
	if (first_rec == (Item *)NULL) // There are no more items in the list
	{
		last_rec = (Item *) NULL; // so the last_rec is also NULL
	}
	ptr -> next = (Item *) NULL; // it doesn't point anywhere
	no_of_items --; // decrement the length of the list
	return (ptr);
}
				
List::Display() // See all the items in the list
{
	int i; // counter
	Item *ptr;

	ptr = first_rec; // start from the beginning of the list

	printf("first_rec = %x, last_rec = %x, no of items = %d\n",
		first_rec,last_rec,no_of_items);
	
	for(i=0; i<no_of_items; i++)	
	{ // show all the elements
		printf("Item No. %d, value = %d, next = %x\n",i+1,
		ptr -> value, ptr -> next);
		ptr = ptr -> next; // go to the next record 
	}
}
