#include "incl.h"

WCItem::WCItem()
{
	next = (WCItem *) NULL;
	WList_ptr = (WList *) NULL;
	WPList_ptr = (WPList *) NULL;
	wcid = 0;
	number_of_workers = 0;
	number_of_work_packets = 0;
}
WCItem::~WCItem()
{
// Nothing. WCItem is like a struct here	
}

WCList::WCList()
{ //Constructor. Initialise.
	first_rec = (WCItem *) NULL;
	last_rec = (WCItem *) NULL;
	no_of_items = 0;

}
WCList::~WCList()
{
//Destructor. Delete each item individually
WCItem *ptr,*next;
	ptr = first_rec;
	while (ptr != (WCItem *)NULL)
	{
		next = ptr->next; // it points to the next item
		delete ptr; // free the space
		ptr = next; // go on to the next, till the end when ptr= NULL
	}
}

WCList::Append(char *wcname,int val)
{
	WCItem *ptr;

	ptr = new WCItem(); // creates the new item
	
	if (ptr == (WCItem *) NULL)
	{
		printf("MEM ALLOCATION ERROR in APPEND. new WCItem() returns NULL\n");
		return (ERROR);
	}
	
	if (no_of_items == 0)
	{
		first_rec = ptr; // first rec points to the new item
		last_rec = first_rec; // there's only one item
	}
	else 
	{ // there are some items on the list
		last_rec->next = ptr; // Append to the end of the list
		last_rec = last_rec->next; //update the last_rec
	} //note last_rec->next = (WCItem *) NULL; is done in the WCItem::WCItem()
		
	last_rec->wcid = val; // put the value in the last item
	strcpy(last_rec->worker_class_name,wcname); // put the name
	no_of_items ++; //in any case increment the no. of items.
	return (OK);
	
}

WCItem * WCList::SearchByName(char *name) 
{
	WCItem *ptr;
	
	ptr = first_rec; // start from the beginning of the list
	while (ptr != (WCItem *) NULL)
	{ // ptr points to something valid	
		if (!strcmp(ptr->worker_class_name,name)) // compare and check
		{
			return ptr; // return the item address - whole record
		}	
		else 
		{ // match not found. go on to the next item and check
			ptr = ptr -> next;
		}
	}
	// traversed the whole WCList and found no item which matches the value
	return (WCItem *) NULL; // Not found
}

WCItem * WCList::Search(int val) 
{ // looks for an item which has wcid = val
	WCItem *ptr;
	
	ptr = first_rec; // start from the beginning of the list
	while (ptr != (WCItem *) NULL)
	{ // ptr points to something valid	
		if (ptr->wcid == val) // compare and check
		{
			return ptr; // return the item address - whole record
		}	
		else 
		{ // match not found. go on to the next item and check
			ptr = ptr -> next;
		}
	}
	// traversed the whole WCList and found no item which matches the value
	return (WCItem *) NULL; // Not found
}

WCList::Delete(int val) 
{
	WCItem *prev, *ptr;

	prev = (WCItem *) NULL; //points to the last item checked. NULL to start
	ptr = first_rec; // points to beginning of the WCList
	if (no_of_items == 0) // there's nothing to delete
		return OK; // fine I've done my job.

	while (ptr != (WCItem *) NULL) // check till there's no more to check
	{
		if(ptr -> wcid == val)
		{
		//matched. Delete this item now	
			if (prev != (WCItem *)NULL) // not the first item
			{
				prev -> next = ptr -> next; //skip this item
				if(prev -> next == (WCItem *) NULL) //last item
				{
					last_rec = prev; //update last rec	
				}
			} 
			else
			{ // this is the first item to be deleted
				first_rec = ptr -> next;
				if (first_rec == (WCItem *) NULL)
				{ // no items on the list
					last_rec = (WCItem *) NULL; //update
				}
			}
			
			// in either case, 
			delete (ptr); // delete the matched item
			no_of_items --; // and decrement the length of the WCList
			return (OK); // fine I've deleted the item
		} 
		// item does not match
		prev = ptr; // now I save the current item in prev and 
		ptr = ptr -> next; //go to the next item in the list
	}
	// No items found to delete
	return (FAILURE);
}


WCItem * WCList::Pop(void)
{ // Just remove the first item from the list
	WCItem *ptr;

	if (no_of_items == 0)
	{
		return (WCItem *) NULL; // There's no item on the list
	}
	
	ptr = first_rec; // take the first item
	first_rec = first_rec -> next; // it now points to the next
	if (first_rec == (WCItem *)NULL) // There are no more items in the list
	{
		last_rec = (WCItem *) NULL; // so the last_rec is also NULL
	}
	ptr -> next = (WCItem *) NULL; // it doesn't point anywhere
	no_of_items --; // decrement the length of the list
	return (ptr);
}
				
WCList::Display() // See all the items in the list
{
	int i; // counter
	WCItem *ptr;

	ptr = first_rec; // start from the beginning of the list

	printf("first_rec = %x, last_rec = %x, no of items = %d\n",
		first_rec,last_rec,no_of_items);
	
	for(i=0; i<no_of_items; i++)	
	{ // show all the elements
		printf("WCItem worker class name = %s, No. %d, wcid = %d, next = %x WPList_ptr=%x,WList_ptr = %x,nWorkers= %d,nWPackets=%d\n",
		ptr->worker_class_name,i+1, ptr -> wcid, ptr -> next,
		ptr->WPList_ptr,ptr->WList_ptr,
		ptr->number_of_workers,ptr->number_of_work_packets);
		ptr = ptr -> next; // go to the next record 
	}
	return (OK);
}
