#include "incl.h"

WItem::WItem()
{
	next = (WItem *) NULL;
	tid = 0;
	packets_pending = -1; // initialise
	// When it asks for a packet, it becomes 0. When it gets packet 1.
}
WItem::~WItem()
{
// Nothing. WItem is like a struct here	
}

WList::WList()
{ //Constructor. Initialise.
	first_rec = (WItem *) NULL;
	last_rec = (WItem *) NULL;
	no_of_items = 0;

}
WList::~WList()
{
//Destructor. Delete each item individually
WItem *ptr,*next;
	ptr = first_rec;
	while (ptr != (WItem *)NULL)
	{
		next = ptr->next; // it points to the next item
		delete ptr; // free the space
		ptr = next; // go on to the next, till the end when ptr= NULL
	}
}

WList::Append(char *wname,int val)
{
	WItem *ptr;

	ptr = new WItem(); // creates the new item
	
	if (ptr == (WItem *) NULL)
	{
		printf("MEM ALLOCATION ERROR in APPEND. new WItem() returns NULL\n");
		return (ERROR);
	}
	
	if (no_of_items == 0)
	{
		first_rec = ptr; // first rec points to the new item
		last_rec = first_rec; // there's only one item
	}
	else 
	{ // there are some items on the list
		last_rec->next = ptr; // Append to the end of the list
		last_rec = last_rec->next; //update the last_rec
	} //note last_rec->next = (WItem *) NULL; is done in the WItem::WItem()
		
	last_rec->tid = val; // put the tid in the last item
	strcpy(last_rec->worker_name,wname); // store the name
	no_of_items ++; //in any case increment the no. of items.
	return (OK);
	
}

WItem * WList::Search(int val) 
{
	WItem *ptr;
	
	ptr = first_rec; // start from the beginning of the list
	while (ptr != (WItem *) NULL)
	{ // ptr points to something valid	
		if (ptr->tid == val) // compare and check
		{
			return ptr; // return the item address - whole record
		}	
		else 
		{ // match not found. go on to the next item and check
			ptr = ptr -> next;
		}
	}
	// traversed the whole WList and found no item which matches the tid
	return (WItem *) NULL; // Not found
}

WItem * WList::SearchNoPending(void) 
{ // Search for that item which has no items pending and wants
  // a packet (Requesting for a packet sets the number of packets
  // pending as 0. Taking a packet sets it as 1, initially its -1

	WItem *ptr;
	
	ptr = first_rec; // start from the beginning of the list
	while (ptr != (WItem *) NULL)
	{ // ptr points to something valid	
		if (ptr->packets_pending == 0) // compare and check
		{
			return ptr; // return the item address - whole record
		}	
		else 
		{ // match not found. go on to the next item and check
			ptr = ptr -> next;
		}
	}
	// traversed the whole WList and found no item which matches the tid
	return (WItem *) NULL; // Not found
}

WList::Delete(int val) 
{
	WItem *prev, *ptr;

	prev = (WItem *) NULL; //points to the last item checked. NULL to start
	ptr = first_rec; // points to beginning of the WList
	if (no_of_items == 0) // there's nothing to delete
		return OK; // fine I've done my job.

	while (ptr != (WItem *) NULL) // check till there's no more to check
	{
		if(ptr -> tid == val)
		{
		//matched. Delete this item now	
			if (prev != (WItem *)NULL) // not the first item
			{
				prev -> next = ptr -> next; //skip this item
				if(prev -> next == (WItem *) NULL) //last item
				{
					last_rec = prev; //update last rec	
				}
			} 
			else
			{ // this is the first item to be deleted
				first_rec = ptr -> next;
				if (first_rec == (WItem *) NULL)
				{ // no items on the list
					last_rec = (WItem *) NULL; //update
				}
			}
			
			// in either case, 
			delete (ptr); // delete the matched item
			no_of_items --; // and decrement the length of the WList
			return (OK); // fine I've deleted the item
		} 
		// item does not match
		prev = ptr; // now I save the current item in prev and 
		ptr = ptr -> next; //go to the next item in the list
	}
	// No items found to delete
	return (FAILURE);
}


WItem * WList::Pop(void)
{ // Just remove the first item from the list
	WItem *ptr;

	if (no_of_items == 0)
	{
		return (WItem *) NULL; // There's no item on the list
	}
	
	ptr = first_rec; // take the first item
	first_rec = first_rec -> next; // it now points to the next
	if (first_rec == (WItem *)NULL) // There are no more items in the list
	{
		last_rec = (WItem *) NULL; // so the last_rec is also NULL
	}
	ptr -> next = (WItem *) NULL; // it doesn't point anywhere
	no_of_items --; // decrement the length of the list
	return (ptr);
}
				
WList::Display() // See all the items in the list
{
	int i; // counter
	WItem *ptr;

	ptr = first_rec; // start from the beginning of the list

	printf("WList first_rec = %x, last_rec = %x, no of items = %d\n",
		first_rec,last_rec,no_of_items);
	
	for(i=0; i<no_of_items; i++)	
	{ // show all the elements
		printf("WItem worker name %s No. %d, tid = %d, packets_pending = %d,next = %x\n",
		ptr->worker_name,i+1,
		ptr -> tid, ptr->packets_pending,ptr -> next);
		ptr = ptr -> next; // go to the next record 
	}
	return (OK);
}
