#include "incl.h"

WPItem::WPItem()
{
	next = (WPItem *) NULL;
	bufid = 0;
	msgtag = 0; // initialise
}
WPItem::~WPItem()
{
// Nothing. WPItem is like a struct here	
}

WPList::WPList()
{ //Constructor. Initialise.
	first_rec = (WPItem *) NULL;
	last_rec = (WPItem *) NULL;
	no_of_items = 0;

}
WPList::~WPList()
{
//Destructor. Delete each item individually
WPItem *ptr,*next;
	ptr = first_rec;
	while (ptr != (WPItem *)NULL)
	{
		next = ptr->next; // it points to the next item
		delete ptr; // free the space
		ptr = next; // go on to the next, till the end when ptr= NULL
	}
}

WPList::Append(int bufid, int msgtag)
{
	WPItem *ptr;

	ptr = new WPItem(); // creates the new item
	
	if (ptr == (WPItem *) NULL)
	{
		printf("MEM ALLOCATION ERROR in APPEND. new WPItem() returns NULL\n");
		return (ERROR);
	}
	
	if (no_of_items == 0)
	{
		first_rec = ptr; // first rec points to the new item
		last_rec = first_rec; // there's only one item
	}
	else 
	{ // there are some items on the list
		last_rec->next = ptr; // Append to the end of the list
		last_rec = last_rec->next; //update the last_rec
	} //note last_rec->next = (WPItem *) NULL; is done in the WPItem::WPItem()
		
	last_rec->bufid = bufid; // store the bufid in the last item
	last_rec->msgtag = msgtag; // store the message tag 
	no_of_items ++; //in any case increment the no. of items.
	return (OK);
	
}

WPItem * WPList::Search(int val) 
{
	WPItem *ptr;
	
	ptr = first_rec; // start from the beginning of the list
	while (ptr != (WPItem *) NULL)
	{ // ptr points to something valid	
		if (ptr->bufid == val) // compare and check
		{
			return ptr; // return the item address - whole record
		}	
		else 
		{ // match not found. go on to the next item and check
			ptr = ptr -> next;
		}
	}
	// traversed the whole WPList and found no item which matches the bufid
	return (WPItem *) NULL; // Not found
}

WPList::Delete(int val) 
{
	WPItem *prev, *ptr;

	prev = (WPItem *) NULL; //points to the last item checked. NULL to start
	ptr = first_rec; // points to beginning of the WPList
	if (no_of_items == 0) // there's nothing to delete
		return OK; // fine I've done my job.

	while (ptr != (WPItem *) NULL) // check till there's no more to check
	{
		if(ptr -> bufid == val)
		{
		//matched. Delete this item now	
			if (prev != (WPItem *)NULL) // not the first item
			{
				prev -> next = ptr -> next; //skip this item
				if(prev -> next == (WPItem *) NULL) //last item
				{
					last_rec = prev; //update last rec	
				}
			} 
			else
			{ // this is the first item to be deleted
				first_rec = ptr -> next;
				if (first_rec == (WPItem *) NULL)
				{ // no items on the list
					last_rec = (WPItem *) NULL; //update
				}
			}
			
			// in either case, 
			delete (ptr); // delete the matched item
			no_of_items --; // and decrement the length of the WPList
			return (OK); // fine I've deleted the item
		} 
		// item does not match
		prev = ptr; // now I save the current item in prev and 
		ptr = ptr -> next; //go to the next item in the list
	}
	// No items found to delete
	return (FAILURE);
}


WPItem * WPList::Pop(void)
{ // Just remove the first item from the list
	WPItem *ptr;

	if (no_of_items == 0)
	{
		return (WPItem *) NULL; // There's no item on the list
	}
	
	ptr = first_rec; // take the first item
	first_rec = first_rec -> next; // it now points to the next
	if (first_rec == (WPItem *)NULL) // There are no more items in the list
	{
		last_rec = (WPItem *) NULL; // so the last_rec is also NULL
	}
	ptr -> next = (WPItem *) NULL; // it doesn't point anywhere
	no_of_items --; // decrement the length of the list
	return (ptr);
}
				
WPList::Display() // See all the items in the list
{
	int i; // counter
	WPItem *ptr;

	ptr = first_rec; // start from the beginning of the list

	printf("first_rec = %x, last_rec = %x, no of items = %d\n",
		first_rec,last_rec,no_of_items);
	
	for(i=0; i<no_of_items; i++)	
	{ // show all the elements
		printf("WPItem No. %d, bufid = %d, msgtag=%d, next = %x\n",i+1,
		ptr -> bufid, ptr->msgtag, ptr -> next);
		ptr = ptr -> next; // go to the next record 
	}
	return (OK);
}
