#include "incl.h"


FItem::FItem(char *name)
{
	next = (FItem *) NULL;
	WCList_ptr = (WCList *) NULL;
	number_of_worker_classes = 0;
	strcpy(farmer_name,name);
}
FItem::~FItem()
{
// Nothing. FItem is like a struct here	
}

FList::FList()
{ //Constructor. Initialise.
	first_rec = (FItem *) NULL;
	last_rec = (FItem *) NULL;
	no_of_items = 0;

}
FList::~FList()
{
//Destructor. Delete each item individually
FItem *ptr,*next;
	ptr = first_rec;
	while (ptr != (FItem *)NULL)
	{
		next = ptr->next; // it points to the next item
		delete ptr; // free the space
		ptr = next; // go on to the next, till the end when ptr= NULL
	}
}

FList::Append(char *name,int val)
{
	FItem *ptr;

	ptr = new FItem(name); // creates the new item
	
	if (ptr == (FItem *) NULL)
	{
		printf("MEM ALLOCATION ERROR in APPEND. new FItem(%s) returns NULL. Value = %d for item to be appended \n",name,val);
		return (ERROR);
	}
	
	if (no_of_items == 0)
	{
		first_rec = ptr; // first rec points to the new item
		last_rec = first_rec; // there's only one item
	}
	else 
	{ // there are some items on the list
		last_rec->next = ptr; // Append to the end of the list
		last_rec = last_rec->next; //update the last_rec
	} //note last_rec->next = (FItem *) NULL; is done in the FItem::FItem()
		
	last_rec->tid = val; // put the tid in the last item
	no_of_items ++; //in any case increment the no. of items.
	return (OK);
	
}

FItem * FList::Search(char *name) 
{ // Here the farmer is searched using the name.
	FItem *ptr;
	
	ptr = first_rec; // start from the beginning of the list
	while (ptr != (FItem *) NULL)
	{ // ptr points to something valid	
		if (!strcmp(ptr->farmer_name,name)) // compare and check
		{
			return ptr; // return the item address - whole record
		}	
		else 
		{ // match not found. go on to the next item and check
			ptr = ptr -> next;
		}
	}
	// traversed the whole FList and found no item which matches the tid
	return (FItem *) NULL; // Not found
}

FItem * FList::SearchByTid(int val)
{ // Here the farmer is searched using the name.
	FItem *ptr;
	
	ptr = first_rec; // start from the beginning of the list
	while (ptr != (FItem *) NULL)
	{ // ptr points to something valid	
		if (ptr->tid == val) // compare and check
		{
			return ptr; // return the item address - whole record
		}	
		else 
		{ // match not found. go on to the next item and check
			ptr = ptr -> next;
		}
	}
	// traversed the whole FList and found no item which matches the tid
	return (FItem *) NULL; // Not found
}

FList::Delete(char *name) 
{
	FItem *prev, *ptr;

	prev = (FItem *) NULL; //points to the last item checked. NULL to start
	ptr = first_rec; // points to beginning of the FList
	if (no_of_items == 0) // there's nothing to delete
		return OK; // fine I've done my job.

	while (ptr != (FItem *) NULL) // check till there's no more to check
	{
		if(!strcmp(ptr->farmer_name,name)) // if found
		{
		//matched. Delete this item now	
			if (prev != (FItem *)NULL) // not the first item
			{
				prev -> next = ptr -> next; //skip this item
				if(prev -> next == (FItem *) NULL) //last item
				{
					last_rec = prev; //update last rec	
				}
			} 
			else
			{ // this is the first item to be deleted
				first_rec = ptr -> next;
				if (first_rec == (FItem *) NULL)
				{ // no items on the list
					last_rec = (FItem *) NULL; //update
				}
			}
			
			// in either case, 
			delete (ptr); // delete the matched item
			no_of_items --; // and decrement the length of the FList
			return (OK); // fine I've deleted the item
		} 
		// item does not match
		prev = ptr; // now I save the current item in prev and 
		ptr = ptr -> next; //go to the next item in the list
	}
	// No items found to delete
	return (FAILURE);
}


FItem * FList::Pop(void)
{ // Just remove the first item from the list
	FItem *ptr;

	if (no_of_items == 0)
	{
		return (FItem *) NULL; // There's no item on the list
	}
	
	ptr = first_rec; // take the first item
	first_rec = first_rec -> next; // it now points to the next
	if (first_rec == (FItem *)NULL) // There are no more items in the list
	{
		last_rec = (FItem *) NULL; // so the last_rec is also NULL
	}
	ptr -> next = (FItem *) NULL; // it doesn't point anywhere
	no_of_items --; // decrement the length of the list
	return (ptr);
}
				
FList::Display() // See all the items in the list
{
	int i; // counter
	FItem *ptr;

	ptr = first_rec; // start from the beginning of the list

	printf("FList first_rec = %x, last_rec = %x, no of items = %d\n",
		first_rec,last_rec,no_of_items);
	
	for(i=0; i<no_of_items; i++)	
	{ // show all the elements
		printf("FItem %s No. %d, tid = %d, number of worker classes = %d,WCList_ptr = %x, next = %x\n",
		ptr->farmer_name,i+1,ptr -> tid,ptr->number_of_worker_classes,
		ptr->WCList_ptr,ptr -> next);
		ptr = ptr -> next; // go to the next record 
	}
	return (OK);
}

int FList::EnrollWorkerClass(char *farm, char *wcname)
{ // returns the worker class id of the worker class, or ERROR or FARM_NOT_CREATED
	FItem *FIptr;
	int ret; // to check the return value of a function

	FIptr = Search(farm); // see if there's a farm with the name
	if (FIptr == (FItem *) NULL)
	{  // not found 
		return (FARM_NOT_CREATED); // Farm not created yet 
	}	
	// found the farm. Its in ptr	
	if (FIptr->WCList_ptr == (WCList *) NULL)
	{ // there are no worker classes associated with the farmer yet
		FIptr -> WCList_ptr = new WCList();
		// create a new list
		if (FIptr->WCList_ptr == (WCList *) NULL)
			return (ERROR); // MALLOC ERROR 
	}
	//Now there is a worker class established. Give back the id
	gwcid ++; // get a unique worker class id, gwcid is a global var
	ret = FIptr -> WCList_ptr-> Append(wcname,gwcid); //create an item
	if (ret == ERROR)  // Append could not create an item
			return (ERROR); // ERROR!! Probably Mem alloc.
		else  
			{
			FIptr -> number_of_worker_classes ++; // added a WC
			return (gwcid); // Hunky dory
			}
	
	
}
