#include "incl.h"
// List of pending requests.

PItem::PItem()
{
	next = (PItem *) NULL;
	opcode = 0;
	tid = 0; 
	memset(farm_name, 0,NAMELEN);
	memset(worker_class_name, 0, NAMELEN); // initialise
}
PItem::~PItem()
{
// Nothing. PItem is like a struct here	
}

PList::PList()
{ //Constructor. Initialise.
	first_rec = (PItem *) NULL;
	last_rec = (PItem *) NULL;
	no_of_items = 0;

}
PList::~PList()
{
//Destructor. Delete each item individually
PItem *ptr,*next;
	ptr = first_rec;
	while (ptr != (PItem *)NULL)
	{
		next = ptr->next; // it points to the next item
		delete ptr; // free the space
		ptr = next; // go on to the next, till the end when ptr= NULL
	}
}

PList::Append(int op,char *fname,char *wcname,int taskid) 
// to the end of the list. Return OK, ERROR
{
	PItem *ptr;

	ptr = new PItem(); // creates the new item
	
	if (ptr == (PItem *) NULL)
	{
		printf("MEM ALLOCATION ERROR in APPEND. new PItem() returns NULL\n");
		return (ERROR);
	}
	
	if (no_of_items == 0)
	{
		first_rec = ptr; // first rec points to the new item
		last_rec = first_rec; // there's only one item
	}
	else 
	{ // there are some items on the list
		last_rec->next = ptr; // Append to the end of the list
		last_rec = last_rec->next; //update the last_rec
	} //note last_rec->next = (PItem *) NULL; is done in the PItem::PItem()
		
	last_rec->opcode = op;  // store all values.
	last_rec->tid = taskid;
	strcpy(last_rec->farm_name,fname);
	strcpy(last_rec->worker_class_name,wcname);

	no_of_items ++; //in any case increment the no. of items.
	return (OK);
	
}

PItem * PList::Search(int val) 
{ // Search for request from task tid pending
	PItem *ptr;
	
	ptr = first_rec; // start from the beginning of the list
	while (ptr != (PItem *) NULL)
	{ // ptr points to something valid	
		if (ptr->tid == val) // compare and check
		{
			return ptr; // return the item address - whole record
		}	
		else 
		{ // match not found. go on to the next item and check
			ptr = ptr -> next;
		}
	}
	// traversed the whole PList and found no item which matches the bufid
	return (PItem *) NULL; // Not found
}

PList::Delete(int val) 
{
	PItem *prev, *ptr;

	prev = (PItem *) NULL; //points to the last item checked. NULL to start
	ptr = first_rec; // points to beginning of the PList
	if (no_of_items == 0) // there's nothing to delete
		return OK; // fine I've done my job.

	while (ptr != (PItem *) NULL) // check till there's no more to check
	{
		if(ptr -> tid == val)
		{
		//matched. Delete this item now	
			if (prev != (PItem *)NULL) // not the first item
			{
				prev -> next = ptr -> next; //skip this item
				if(prev -> next == (PItem *) NULL) //last item
				{
					last_rec = prev; //update last rec	
				}
			} 
			else
			{ // this is the first item to be deleted
				first_rec = ptr -> next;
				if (first_rec == (PItem *) NULL)
				{ // no items on the list
					last_rec = (PItem *) NULL; //update
				}
			}
			
			// in either case, 
			delete (ptr); // delete the matched item
			no_of_items --; // and decrement the length of the PList
			return (OK); // fine I've deleted the item
		} 
		// item does not match
		prev = ptr; // now I save the current item in prev and 
		ptr = ptr -> next; //go to the next item in the list
	}
	// No items found to delete
	return (FAILURE);
}


PItem * PList::Pop(void)
{ // Just remove the first item from the list
	PItem *ptr;

	if (no_of_items == 0)
	{
		return (PItem *) NULL; // There's no item on the list
	}
	
	ptr = first_rec; // take the first item
	first_rec = first_rec -> next; // it now points to the next
	if (first_rec == (PItem *)NULL) // There are no more items in the list
	{
		last_rec = (PItem *) NULL; // so the last_rec is also NULL
	}
	ptr -> next = (PItem *) NULL; // it doesn't point anywhere
	no_of_items --; // decrement the length of the list
	return (ptr);
}
				
PList::Display() // See all the items in the list
{
	int i; // counter
	PItem *ptr;

	ptr = first_rec; // start from the beginning of the list

	printf("first_rec = %x, last_rec = %x, no of items = %d\n",
		first_rec,last_rec,no_of_items);
	
	for(i=0; i<no_of_items; i++)	
	{ // show all the elements
		if(ptr->opcode == GET_WORKER_CLASS_ID)
			printf("PItem No. %d, opcode = GET_WORKER_CLASS_ID, tid =%d, farm_name = %s, worker_class_name = %s ,next = %x\n",i+1,
			ptr -> tid, ptr->farm_name,ptr->worker_class_name, 
			ptr -> next);
		if(ptr->opcode == INIT_WORKER_CLASS)
			printf("PItem No. %d, opcode = INIT_WORKER_CLASS, tid =%d, farm_name = %s, worker_class_name = %s ,next = %x\n",i+1,
			ptr -> tid, ptr->farm_name,ptr->worker_class_name, 
			ptr -> next);

		ptr = ptr -> next; // go to the next record 
	}
	return (OK);
}
