#include "incl.h"
#include <pvm3.h>

int	ProcessSendWorkPacket(int ctid, int wcid)
{
int 	msgbufid,sbufid;
WCItem	*wcitem; // worker class item
WPItem	*wpitem; // work packet item
WItem 	*witem; // worker item	

	
	// We have the farmer's tid (ctid) and the worker class id to which
	// it wants to send a work packet to.
	// First receive the work packet

	msgbufid = pvm_mkbuf(PvmDataDefault);
	ASSERT(msgbufid); // Check if < 0

	DEBUG("ProcSndWP new msgbufid = %d\n",msgbufid);
	info = pvm_setrbuf(msgbufid); 
	ASSERT(info); // Check if info < 0

	msgbufid = pvm_recv(ctid, DEF_MSGTAG); // receive the wp from the farmer
	ASSERT(msgbufid); // Check if < 0

	msgbufid = pvm_setrbuf(0);
	ASSERT(msgbufid); // Check if < 0
	DEBUG("SR: SndWP received from farmer bufid = %d\n",msgbufid);
	
/* we have to get the worker class item using the wcid that has been received */

	wcitem = SearchForWC(wcid); // gets the wcitem.
	// Search the worker class list for an item which has the same wcid
	if (wcitem == (WCItem *) NULL)
		return (ERROR); // wcid not found!!! ERROR!!!
	// Now we have the wcitem which has pointers to the work packet and
	// worker task lists

	/* Algorithm for the server is that it checks to see if there are
	 any tasks which are waiting for a work packet (packets_pending == 0)
	and if there are then send the work packet to the task. If there 
	aren't any tasks waiting for the work packet then the work packet
	is appended to the end of the list of work packets to be sent.  */

	witem = wcitem->WList_ptr->SearchNoPending(); // look for an idle task

	if (witem == (WItem *) NULL)
	{
		DEBUG("SR: SendWP:No idle worker Append to WPList\n");
		// There's no idle worker in the entire list
		// Append the work packet to the WPList
			
		if(wcitem->WPList_ptr == (WPList *) NULL)
		{ // there are no work packets and list
			wcitem->WPList_ptr = new WPList(); // create a WPlist
			if (wcitem->WPList_ptr == (WPList *) NULL)
				return (ERROR); // MALLOC ERROR
		}
		// So, the list of work packets is established.

		wcitem->WPList_ptr->Append(msgbufid, DEF_MSGTAG); 
		
		//After appending the item to the end of the list 
		return (OK); // fine! job done
	}
	else
	{
		DEBUG("SR: SendWP:Found worker -> sending WP just recd\n");
		// Yes! There is at least one worker who needs the work
		// packet that has been received. Send it to the worker
		
		DEBUG("SR: SndWP: Giving worker bufid = %d\n", msgbufid);
		info = pvm_setsbuf(msgbufid); // put recv buffer
			// for sending to the worker
		ASSERT(info); // Check if info < 0

	        info = pvm_send(witem->tid, DEF_MSGTAG);
	        // send the worker packet to the worker
		ASSERT(info); // Check if info < 0
	
	        info = pvm_freebuf(msgbufid); // clear the buffer
		ASSERT(info); // Check if info < 0
		 
		// So the work packet has reached the worker
		// Now take the worker item and marked it as not-idle	
		witem ->packets_pending++; // increment the packets

		// That's it now the worker task is processing the wp!
		return (OK);
	}
}	



int	ProcessRecvWorkPacket(int ctid, int wcid)
{
int 	msgbufid,sbufid,ret;
WCItem	*wcitem; // worker class item
WPItem	*wpitem; // work packet item
WItem 	*witem; // worker item	

	// the worker task id is in ctid (client task id), and its in wcid	
	// worker class and it is associated with the farmer whose id is ftid
	
	// Look for a work packet. Before we can do that we need to get to
	// the worker class item
	// First find the farmer
#ifdef OLD
	fitem = farm->Search(farm_name); // search by farmer name
	if(fitem == (FItem *) NULL)
	{ // FARM is not present. So, it has terminated. Since the worker
	  // was busy doing something else, it had not received the 
	  // notification. Send it now.

		 sbufid = pvm_mkbuf(PvmDataDefault); // makes buffer
		 ASSERT(sbufid); // Check if sbufid < 0

       		 info = pvm_setsbuf(sbufid); // sets this one as
        	 // the active send buffer. It'll be used
		 ASSERT(info); // Check if info < 0

		 ret = FARM_HAS_TERMINATED ; 
        	 info = pvm_pkint(&ret, 1, 1); // 1 item, stride = 1
		 ASSERT(info); // Check if info < 0

		 info = pvm_send(witem -> tid, FARM_HAS_TERMINATED);
        	 // send the message to each worker waiting for it
		 ASSERT(info); // Check if info < 0

        	 info = pvm_freebuf(sbufid);
		 ASSERT(info); // Check if info < 0

		 return (OK);
	}
	// So, now we have the farmer item in fitem
	// Next search for the worker class item
	wcitem = fitem->WCList_ptr->Search(wcid); 
#endif

	wcitem = SearchForWC(wcid);
	if(wcitem == (WCItem *) NULL)
	{ // FARM is not present. So, it has terminated. Since the worker
	  // was busy doing something else, it had not received the 
	  // notification. Send it now. Note : ProcessFarmTerminate calls
	  // ResetWCCache(wcid) which makes this wcitem NULL.

		 sbufid = pvm_mkbuf(PvmDataDefault); // makes buffer
		 ASSERT(sbufid); // Check if sbufid < 0

       		 info = pvm_setsbuf(sbufid); // sets this one as
        	 // the active send buffer. It'll be used
		 ASSERT(info); // Check if info < 0

		 ret = FARM_HAS_TERMINATED ; 
        	 info = pvm_pkint(&ret, 1, 1); // 1 item, stride = 1
		 ASSERT(info); // Check if info < 0

		 info = pvm_send(witem -> tid, FARM_HAS_TERMINATED);
        	 // send the message to each worker waiting for it
		 ASSERT(info); // Check if info < 0

        	 info = pvm_freebuf(sbufid);
		 ASSERT(info); // Check if info < 0

		 return (OK);
	}


	// Search the worker class list for an item which has the same wcid
	if (wcitem == (WCItem *) NULL)
		return (ERROR); // wcid not found!!! ERROR!!!
	witem = wcitem->WList_ptr->Search(ctid); // worker task id search
	if (witem == (WItem *) NULL)
		return (ERROR);
	// Now we have the wcitem which has pointers to the work packet and
	// worker task lists

	/* Now the algorithm for the server is that it pops an item from the
	work packet list. If there's no work packet to be processed, then,
	we get the worker item from the worker list and set the packets_pending
	field to 1 and send the work packet (just popped) to the worker task.
	After this we do house keeping for the popped work packet (free stuff)
	
	In case there are no work packets (pop gives null), then we set the
	worker items packets_pending field to 0. So that when a work packet
	does come, it will search for those tasks which have no packets pending
	and get this task. Then it will be sent the work packet
	*/


	// At this stage, we have wcitem and witem which are valid

	// Now we pop a work packet
	if(wcitem->WPList_ptr == (WPList *) NULL)
	{ // there are no work packets and list
		wcitem->WPList_ptr = new WPList(); // create a WPlist
		if (wcitem->WPList_ptr == (WPList *) NULL)
			return (ERROR); // MALLOC ERROR
	}
	// So, the list of work packets is established.

	wpitem = wcitem->WPList_ptr->Pop();


	if (wpitem == (WPItem *) NULL)
	{
		// no work packet to be processed
		DEBUG("SR:RecvWP : No WP on list Mark the WItem and leave\n");

		witem->packets_pending = 0; // Set the field to zero
		return (OK); // thats all!
	}
	else
	{

		DEBUG("SR:RecvWP : Found WP on list.Giving it to worker\n");
		// Work packet is found! Give it to the worker
		DEBUG("SR: RcvWP giving from wplist bufid = %d\n",wpitem->bufid);
		info = pvm_setsbuf(wpitem->bufid); // set this for sending
		ASSERT(info); // Check if info < 0

		info = pvm_send(ctid, DEF_MSGTAG); // send the wp	
		ASSERT(info); // Check if info < 0
		if (info < 0) return (ERROR);
	
		// Housekeeping work
		info = pvm_freebuf(wpitem->bufid); // free the buffer
		ASSERT(info); // Check if info < 0
	
		delete wpitem; // delete the storage for the work packet item

		witem->packets_pending = 1; // Its processing packets

		return (OK); // Thats it
	}
}

int	ProcessLeaveFarm(int ctid, int ftid, int wcid)
{
int 	msgbufid,sbufid,ret;
FItem 	*fitem; // farmer item
WCItem	*wcitem; // worker class item

	// the worker task id is in ctid (client task id), and its in wcid	
	// worker class and it is associated with the farmer whose id is ftid
	
	// Look for a work packet. Before we can do that we need to get to
	// the worker class item
#ifdef OLD
	// First find the farmer
	fitem = farm->SearchByTid(ftid); // search by farmer task id
	if(fitem == (FItem *) NULL)
	{ // FARM is not present. So, it has terminated. 
	  // So no need to delete anything. Its already done. OK
		 SendMessageToWorker(OK,ctid);
		 return (OK);
	}
	// So, now we have the farmer item in fitem
	// Next search for the worker class item
	wcitem = fitem->WCList_ptr->Search(wcid); 
	if (wcitem == (WCItem *) NULL)
	{	//Worker class id not found. Send an error notification to the 
		//worker waiting for the reply.
		SendMessageToWorker(ERROR, ctid);
		return (ERROR); // wcid not found!!! ERROR!!!
	}
#endif
	wcitem = SearchForWC(wcid);
	// Search the worker class list for an item which has the same wcid
	if (wcitem == (WCItem *) NULL)
	{ // farm is not present (only ProcessFarmTerminate wipes this!).
	  // So no need to delete anything. Its already done. OK
		 SendMessageToWorker(OK,ctid);
		 return (OK);
	}
	 
	// So, the worker class is found.
	// Now we have the wcitem which has pointers to the work packet and
	// worker task lists

	/* Now the algorithm for the server is that it just deletes the
	   worker task entry from this worker class. If its deleted properly
	   an OK message is sent. Else an error notification is sent out to
	   the worker task. */
	ret = wcitem->WList_ptr->Delete(ctid); // worker task id delete
	if (ret != OK)
	{
		SendMessageToWorker(ERROR, ctid);
	}
	else
	{	
		SendMessageToWorker(OK, ctid); // Send an OK
	}
	return (OK); // successful operation. Go to next request.
}

int SendMessageToWorker(int message, int worker_tid)
{
int sbufid;
		 sbufid = pvm_mkbuf(PvmDataDefault); // makes buffer
		 ASSERT(sbufid); // Check if sbufid < 0

       		 info = pvm_setsbuf(sbufid); // sets this one as
        	 // the active send buffer. It'll be used
		 ASSERT(info); // Check if info < 0

        	 info = pvm_pkint(&message, 1, 1); // 1 item, stride = 1
		// pack the integer message (like OK, ERROR)
		 ASSERT(info); // Check if info < 0

		 info = pvm_send(worker_tid, DEF_MSGTAG);
		// Send this message back to the worker
		 ASSERT(info); // Check if info < 0

        	 info = pvm_freebuf(sbufid);
		 ASSERT(info); // Check if info < 0

		 if(info < 0)
			return (ERROR);
		 else
		 	return (OK);
}

WCItem * SearchForWC( int wcid)
{ // this returns the worker class item using the index wcid into the cache.
	return (wccache[wcid]);
}

void ResetWCCache(int wcid)
{ // resets the cache entry containing wcid
	wccache[wcid] = (WCItem *) NULL; 
	DEBUG("Reset the cache item %d\n", wcid);
	return;
}

