/*************************************************************************
**	Project : CIS 631 project.	(test matrix multiply program)  **
**	Author 	: Sameer Shende (540-45-6642)				**
**	Location: /home/grads/sameer/cis631/proj/test			**
**	File	: mat.c							**
**	Description : This program multiplies two n x n float matrices  **
**									**
*************************************************************************/

#include <stdio.h>
#include <malloc.h>
#include <sys/types.h>
#include <sys/timeb.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <math.h>
#include <stdlib.h>
#include "mat.h"
#include <pvm3.h>
#include <pvmfarm.h>

float	**A, **B;	/* The two matrices (global)	 */
float	**C; 		/* The result matrix C=AxB	 */



extern int N, cc, *tid; // globals declared in other file


/* Usage : t1 <matrix size >	*/

int oldmain(int argc,char **argv)
{
int 	size,i,j;

	if (argc > 1)
		size = atoi(argv[1]);	/* 2nd argument to the prog. is size */
	else
		size = DEFAULT ;	/* Use default if invoked w/o size   */
	

	initialise_matrices(size); /* Allocate memory for matrices and 
					put values in A and B matrices. */
	
	multiply_matrices(size,1);   /* C = A.B Multiplication.	*/

 	display_statistics(size);   /* from getrusage command */

#ifdef DEBUG
	printf("A[][]\n");
	display_matrix(A,size);	/* Debugging options to see if multiplied 
					correctly. */

	printf("B[][]\n");
	display_matrix(B,size);
	
	printf("C[][]\n");
	display_matrix(C,size);
#endif

	free_matrices(size);	/* Free the memory before exiting. */
	exit(0);		/* clean exit. Hunky Dory. 	*/
}

/******************************************************************************
** initialise_matrices: routine for doing memory allocation and initialisation*
**	argument      : size of the matrix.			              *
**	affects       : global var. A, B and C.				      *
**	return value  : 1. or exits with an error.	 		      *
******************************************************************************/	
int initialise_matrices(int n)
{
	int i,j;	/* counters for for loop */
/* Here we allocate an array of pointers. First Dimension.*/
	
	if((A = (float **) malloc( n*4 )) == (float **) NULL)
	{
		perror ("Malloc Error in *A allocation");
		exit(1);
	}
	/* A is a 2D array. *A represents the contents of the 1st dim. ary */

	for (i =0; i< n; i++)
	{
		if((A[i] = (float *) malloc (n * sizeof(float)))== (float *) NULL)
		{
			perror ("Malloc Error in A[i] allocation");
			exit(1);
		}
	}

	if((B = (float **) malloc( n *4)) == (float **) NULL)
	{
		perror ("Malloc Error in *B allocation");
		exit(1);
	}
	/* B is a 2D array. *B represents the contents of the 1st dim. ary */

	for (i =0; i< n; i++)
	{
		if((B[i] = (float *) malloc (n * sizeof(float)))== (float *) NULL)
		{
			perror ("Malloc Error in B[i] allocation");
			exit(1);
		}
	}
		
	if((C = (float **) malloc( n *4)) == (float **) NULL)
	{
		perror ("Malloc Error in *C allocation");
		exit(1);
	}
	/* C is a 2D array. *C represents the contents of the 1st dim. ary */

	for (i =0; i< n; i++)
	{
		if((C[i] = (float *) malloc (n * sizeof(float)))== (float *) NULL)
		{
			perror ("Malloc Error in C[i] allocation");
			exit(1);
		}
	}
/* Initialisation section. Contents of A and B are random real numbers between
0 and 1. C matrix is initialised to all zeroes. */
	for(i=0;i<n;i++)
		for(j=0;j<n;j++)
		{
			A[i][j] = i;
		/* random()/2.147483647e9; */ /* the max value */
	
			B[i][j] = j ;
			C[i][j] = 0;
		}		
}
	
/******************************************************************************
** 	 free_matrices: routine for freeing the matrices. Called at the end.  *
**	 argument     : size of the matrix.			              *
**       affects      : global var. A, B and C.				      *
**	 return value : 1.					              *	
******************************************************************************/	
int free_matrices(int n)
{
	int i; /* counter*/
		
	for (i=0;i<n;i++)
	{
		free(A[i]); /* free the rows */
		free(B[i]);
		free(C[i]);
	}
	free(A); /* then free the array containing the pointers to the rows.*/
	free(B);
	free(C);
	
	return(1);
}
 
/******************************************************************************
**   multiply_matrices: routine for matrix multiplication. 	              *	
**	 argument     : size of the matrix.			              *
**       affects      : global var. C. After this  C contains A.B	      *
**	 return value : 1.					              *	
******************************************************************************/	
int multiply_matrices(int n, int row)
{
int i,j,k;	/* Counters */
	
		for (j=0 ;j < n; j++)
		{	
			C[row][j] = 0; /* for every el. C[i][j] */	
			for (k=0;k<n;k++)
				C[row][j] += A[row][k] * B[k][j];
				/*contains product*/
		}
	return(1);
}
	
/******************************************************************************
** 	display_matrix: routine for displaying a matrix. For debugging.	      *
**	arguments     : the matrix (mat) and the size of the matrix(n).       *
**      affects       : nothing. It just displays the matrix.		      *
**	return value  : 1.					              *	
******************************************************************************/	
int display_matrix(float **mat,int n)
{
int	i,j;
	
	for(i=0;i<n;i++)
	{
		printf("[");
		for(j=0;j<n;j++)
		{
			printf("%g ", mat[i][j]);
		}
		printf("]\n");
	}
	return(1);
}

/******************************************************************************
**  display_statistics: routine for showing time. While debugging it also     *
**			shows lots of other interesting stuff from getrusage  *
**	 argument     : size of the matrix.			              *
**       affects      : nothing					              *	
**	 return value : 1.					              *	
******************************************************************************/	
int display_statistics(int n)
{
struct rusage ru;
int	ret;
float	time_taken;
	
	if((ret = getrusage(RUSAGE_SELF,&ru)) < 0)
	{
		perror ("GETRUSAGE Error");
		exit(1);
	}
	
	time_taken = ru.ru_utime.tv_sec + ru.ru_stime.tv_sec + 
			(ru.ru_utime.tv_usec + ru.ru_stime.tv_usec)*1e-6;
	printf("%d	%g\n",n,time_taken);
#ifdef DEBUG
	printf("user time used = %ld sec, %ld usec\n", 
		ru.ru_utime.tv_sec, ru.ru_utime.tv_usec);	
	printf("system time used = %ld sec, %ld usec\n", 
		ru.ru_stime.tv_sec, ru.ru_stime.tv_usec);	
	printf("page faults not requiring physical I/O = %d\n",ru.ru_minflt);
	printf("page faults requiring physical I/O     = %d\n",ru.ru_majflt);
	printf("swaps = %d\n",ru.ru_nswap);
	printf("block input operations = %d\n", ru.ru_inblock);
	printf("block output operations = %d\n", ru.ru_oublock);
	printf("voluntary context switches = %d\n", ru.ru_nvcsw);
	printf("involuntary context switches = %d\n", ru.ru_nivcsw);
#endif
	return(1);

}

/* EOF */
