#include <pvm3.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/timeb.h>
#include <pvmfarm.h>
#include "mat.h" // header file containing constants, etc.

int *tid, size, N,cc; 
float *data;
extern float **C;
int display_time_taken(struct timeb tp1, struct timeb tp2);
int kill_all(void); 
// There are N workers and size of the matrix to be solved is size
main(int argc, char **argv)
{
	int wcid,i,val,sbufid, ret,j,indx;
	struct timeb tp1, tp2;


	if(argc > 1) size = atoi(argv[1]);
	else	     size = DEFAULT;

	data = new float[size]; 

	initialise_matrices(size);
	start_workers(argv); // initiate tasks

/* Send the row no. to be calculated */
	ftime(&tp1);  // see the time
	for(i=0; i< size; i++)
	{
		sbufid = pvm_mkbuf(PvmDataDefault);
		ASSERT(sbufid);
	
		cc = pvm_setsbuf(sbufid); // set this as the sending buffer
		ASSERT(cc);

		cc = pvm_pkint(&i, 1, 1); // pack the data in it
		ASSERT(cc);
		indx = i % N; 
	
		cc = pvm_send(tid[indx],DEFAULT); // send the wp
		ASSERT(cc);
		cc = pvm_freebuf(sbufid); // free up the buffer
		ASSERT(cc);
	}
	
	for(i=0;i<size;i++)
	{
		cc = pvm_recv(-1, DEFAULT);
		ASSERT(cc);
		
		cc = pvm_upkint(&val,1,1); // remove the value
		ASSERT(cc);
		cc = pvm_upkfloat(data, size, 1); // get the row
//		printf("FARMER Received %d, iteration = %d\n", val,i);
		for(j=0;j<size;j++)
			C[val][j] = data[j];
	}
	ftime(&tp2);
	display_time_taken(tp1, tp2);
	//display_matrix(C,size);
	kill_all();
	cc = pvm_exit();
	ASSERT(cc);
}



int display_time_taken(struct timeb tp1, struct timeb tp2)
{
	float time_taken;
/*
	printf("tp1.time = %d, tp1.millitm = %d, tp2.time = %d, tp2.millitm = %d\n",
	tp1.time, tp1.millitm, tp2.time, tp2.millitm);
*/
	time_taken = (tp2.time - tp1.time) + (tp2.millitm - tp1.millitm) * 1e-3;
	printf("matm size %d N %d time taken = %g\n", size, N,time_taken);
	printf("%d	%g\n", size, time_taken);
}

int kill_all(void)
{
	int i;
	for(i = 0; i< N; i++)
		pvm_kill(tid[i]);
}

int start_workers(char **argv)
{
	N = 6; // Number of tasks that will be started here.
	tid = new int[N];
	cc = pvm_spawn("mats", &argv[1], 0, "majestix", 1, &tid[0]);
	ASSERT(cc);
	cc = pvm_spawn("mats", &argv[1], 0, "atlantix", 1, &tid[1]);
	ASSERT(cc);
#ifdef X
#endif
	cc = pvm_spawn("mats", &argv[1], 0, "obelix", 1, &tid[2]);
	ASSERT(cc);
	cc = pvm_spawn("mats", &argv[1], 0, "pacifix", 1, &tid[3]);
	ASSERT(cc);
	cc = pvm_spawn("mats", &argv[1], 0, "geriatrix", 1, &tid[4]);
	ASSERT(cc);
	cc = pvm_spawn("mats", &argv[1], 0, "idiotix", 1, &tid[5]);
	ASSERT(cc);
	
	return (1); 
}

// matf = matw (farmer worker - pvmfarm)
// matm = mats (master slave - no farming) 
